#!/usr/bin/env python
# -*- coding: utf-8 -*-
################################################################################
# Eole NG - 2008
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# upgrade_distrib.py
#
# script de mise à jour du serveur dans zephir après changement de distribution
#
# - vérification du module actuel dans la base zephir
# - migration dans la base vers le module installé sur le serveur
#
################################################################################

import sys, os, getpass, shutil, time, socket
from configobj import ConfigObj
from pyeole.ihm import print_orange

registered = False
old_version = 4
new_version = 5

def message_genconfig(zephirok=False):
    print ""
    print_orange("Pour finaliser la procédure, veuillez :")
    print_orange("- redémarrer le serveur")
    if not zephirok:
        print_orange("- mettre à jour la configuration (par exemple via gen_config)")
    print_orange("- lancer la commande 'instance'")
    print ""

try:
    from zephir.lib_zephir import *
    from zephir.zephir_conf import zephir_conf
    assert int(config.id_serveur) > 0
    registered = True
except:
    # serveur non enregistré sur zephir, on passe cette étape
    print("""Pas d'enregistrement dans une base Zephir détecté.""")
    message_genconfig()

def upgrade():
    """migration du serveur dans zephir"""
    print_orange("""
Cette fonctionnalité nécessite un compte ayant les permissions suivantes
dans l'application Zephir gérant ce serveur (%s):""" % zephir_conf.adresse_zephir)
    print """
- Lecture
- Actions sur les clients (avec ou sans modification de configuration) ou enregistrement
- Ecriture sur les serveurs et les modules (ou Migration de serveur + Ecriture (modules))

    """
    authentified, data = get_pwd(zephir_conf.adresse_zephir, 7080)
    if authentified == False:
        return data
    else:
        proxy = data
    id_new_mod = 0
    old_module = ""
    id_variante = ""
    # récupération du module installé sur la machine
    try:
        module = creole_vars.get_creole('eole_module', '')
        version = creole_vars.get_creole('eole_version', '')
        assert module, version
    except:
        raise Exception('impossible de trouver le nom de module')
    module_local = '%s-%s'%(module, version)
    # on récupère les informations sur le serveur dans la base zephir
    try:
        info_serveur = convert(proxy.serveurs.get_serveur(config.id_serveur))
    except xmlrpclib.ProtocolError:
        return """Erreur de permissions ou Zephir non disponible"""
    except socket.error, e:
        return """Erreur de connexion au serveur Zephir (%s)""" % str(e)
    if info_serveur[0] == 0:
        return "Erreur de recherche du serveur dans la base zephir : "+str(info_serveur[1])
    # on vérifie le n° de module du serveur dans la base zephir
    id_module = info_serveur[1][0]['module_actuel']
    modules = convert(proxy.modules.get_module())
    if modules[0] == 0:
        return "Erreur de recherche du module dans la base zephir : "+str(modules[1])
    # on détermine le numéro dans zephir du module installé
    for mod in modules[1]:
        if mod['libelle'] == module_local:
            id_new_mod = mod['id']
        if mod['id'] == id_module:
            old_module = mod['libelle']
    # mise à jour de /etc/sudoers
    update_sudoers()
    # gestion de la configuration et migration du serveur dans l'application Zéphir
    #
    # Si la configuration de migration n'a pas été générée au préalable, on prévient l'utilisateur
    # qu'il devra valider la configuration localement, puis lancer enregistrement_zephir en sauvegardant
    # la configuration sur Zéphir
    #
    # Dans le cas inverse, on effectue une descente de la configuration de migration et des données de variante
    # Puis, on fait une sauvegarde complète pour remonter les fichiers locaux
    zephir_conf_ok = False
    try:
        # détection de l'état de migration dans Zéphir
        code, res = convert(proxy.serveurs.get_status(config.id_serveur))
        if code == 1 and res['migration_ok'] == 0:
            # une nouvelle configuration est présente sur Zéphir
            zephir_conf_ok = True
    except xmlrpclib.ProtocolError:
        return """Erreur de permissions ou zephir non disponible"""
    except socket.error:
        return """Erreur lors de la détection de l'état de migration dans l'application zephir"""
    if not zephir_conf_ok:
        print ""
        print_red("Attention ! La configuration de migration n'a pas été préparée sur le serveur Zéphir")
        print ""
        print_orange("Veuillez sélectionner une variante parmi celles proposées :")
        id_variante = get_module_var(proxy, id_new_mod)
    try:
        assert id_new_mod != 0 and old_module != ""
        # on indique à migrate_serveur qu'on est en mode Upgrade-Auto
        code, res = convert(proxy.serveurs.migrate_serveur(config.id_serveur, {}, id_new_mod, id_variante, True))
        if code == 0:
            return "Erreur lors de la migration dans l'application Zéphir: %s" % res
        # affichage de la variante du serveur migré
        try:
            code_ret, data = convert(proxy.serveurs.get_serveur(config.id_serveur))
            if code_ret == 1:
                code_ret, var_info = convert(proxy.modules.get_variante(data[0]['variante']))
                if code_ret == 1:
                    print_green("\nLe serveur a été basculé dans Zéphir sur la variante %d (%s) du module %s" \
                    % (data[0]['variante'], var_info[0]['libelle'], module_local))
        except:
            print_red("\n!! Erreur lors de la vérification des données du serveur sur Zéphir !!")

    except xmlrpclib.ProtocolError:
        return """Erreur de permissions ou zephir non disponible"""
    except socket.error:
        return """Erreur lors de la migration dans l'application zephir"""
    # on regarde si la migration de données (fichiers divers) est possible 
    try:
        code_ret, available = migrate_data(proxy, config.id_serveur, check=True)
    except socket.error, e:
        exit_err("Erreur lors de l'appel au serveur Zéphir (%s)" % str(e))
    if code_ret == 1 and available == True:
        print_orange("""\nCertaines données utilisateur peuvent être récupérées\n(fichiers divers définis sur Zéphir)\n""")
        rep_mig = raw_input("""Voulez vous migrer ces données (O/N) ?""")
        if rep_mig.upper() in ['O','OUI']:
            try:
                code_ret, infos = migrate_data(proxy, config.id_serveur)
            except socket.error, e:
                exit_err("Erreur lors de l'appel au serveur Zéphir (%s)" % str(e))
            if code_ret == 0:
                print_red("!! erreur lors de la migration des données suivantes :\n %s" % infos)
    # après migration, on descend les données et la configuration migrées
    if not zephir_conf_ok:
        print ('\nRécupération des données de variante sur Zephir')
    else:
        print ('\nRécupération de la configuration et des données de variante sur Zephir')
    res = proxy.uucp.configure(config.id_serveur)
    if res[0] == 1:
        res = os.system("%s/scripts/zephir_client call >/dev/null" % zephir_dir)
    else:
        exit_err("\n erreur lors de la récupération des données de configuration : %s \n" % res[1])
    print ("** attente de la mise en place des fichiers **")
    # on attend que les fichiers soient mis en place
    # (timeout de 20 secondes au cas ou la procédure échoue)
    wait = 0
    while wait < 40:
        time.sleep(0.5)
        wait += 1
        if os.path.exists(public_dir+'/config-zephir.tar.ok'):
            wait = 100
    get_paqs_from_logs()
    if wait != 100:
        exit_err("!!  La mise en place de la configuration s'est mal terminée !!")
    if zephir_conf_ok:
        # sauvegarde des données locales sur zephir en fin de procédure
        print ('\nSauvegarde de la configuration actuelle sur Zephir')
        res = os.system('/usr/share/zephir/scripts/zephir_client del_lock >/dev/null')
        res = os.system('/usr/share/zephir/scripts/zephir_client save_files >/dev/null')
        message_genconfig(True)
    else:
        message_genconfig()
        # si pas de configuration définie, la sauvegarde devra être faite manuellement
        # après génération de la nouvelle configuration avec gen_config
        print_orange("Puis, utilisez une de ces 2 méthodes pour remonter la configuration sur Zéphir :")
        print_orange(" - demander une sauvegarde de configuration dans l'application Zéphir")
        print_orange("  ou")
        print_orange(" - lancez '/usr/share/zephir/scripts/zephir_client save_files' sur ce serveur")
        print ""
    return ""

def upgrade_modules(version_src, version_dst):
    """fonction de copie des variantes dans le cas d'un upgrade du serveur zephir
    """
    from zephir import config
    from zephir.web.html.erreur import u, convert

    authentified, data = get_pwd(config.ADRESSE_ZEPHIR, config.PORT_ZEPHIR)
    if authentified == False:
        return False, data
    else:
        proxy = data

    # récupération des variantes sources (2.0)
    print "Recherche des variantes sources existantes"
    res, modules_actuel = convert(proxy.modules.get_module())
    if res == 0:
        exit_err('Erreur de lecture des modules')
    mods = {}
    variantes = {}
    variantes_dst = {}

    res, variantes_ori = convert(proxy.modules.get_variante())
    if res == 0:
        exit_err('Erreur de lecture des variantes')

    for module in modules_actuel:
        tag = ''
        mod_name = module['libelle'][:module['libelle'].rindex('-')]
        if module['version'] == version_src:
            tag = 'src'
            # recherche des variantes existantes
            for var in variantes_ori:
                if var['module'] == module['id']:
                    if var['libelle'] != 'standard':
                        if variantes.has_key(module['id']):
                            variantes[module['id']].append((var['id'],var['libelle'],var['owner']))
                        else:
                            variantes[module['id']] = [(var['id'],var['libelle'],var['owner'])]

        elif module['version'] == version_dst:
            mod_name = module['libelle']
            mod_name = mod_name[:mod_name.rindex('-')]
            tag = 'dst'
            # recherche déjà migrées
            for var in variantes_ori:
                if var['module'] == module['id']:
                    if var['libelle'] != 'standard':
                        if variantes_dst.has_key(module['id']):
                            variantes_dst[module['id']].append(var['libelle'])
                        else:
                            variantes_dst[module['id']] = [var['libelle']]
        # stockage de la correspondance des modules source/dest
        if tag != '':
            if mod_name not in mods:
                mods[mod_name] = {tag:module['id']}
            else:
                mods[mod_name][tag] = module['id']

    # transfert des données des modules 2.0
    for modname, data in mods.items():
        if data.has_key('src') and data.has_key('dst'):
            # fichiers des valeurs par défaut du module
            module_eol_src = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'modules',str(data['src']),'module.eol')
            if os.path.isfile(module_eol_src):
                module_eol_dst = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'modules',str(data['dst']),'module.eol')
                print "Reprise des valeurs par défaut pour le modules %s" % modname
                shutil.copy(module_eol_src, module_eol_dst)
            if data['src'] in variantes:
                for var in variantes[data['src']]:
                    id_var, libelle, owner = var
                    copy_needed = True
                    if libelle in variantes_dst.get(data['dst'],[]):
                        # si la variante existe déjà dans le module destination, on ne fait rien
                        copy_needed = False
                    if copy_needed:
                        # libelle de la nouvelle variante
                        code, res = proxy.modules.copy_variante(data['dst'],id_var,u(libelle),"",True)
                        print "Création de la variante pour %s : %s-%s" % (modname, id_var, libelle)
    return True, "ok"

def get_pwd(addr, port):
    """lecture d'un login/passwd pour l'application zephir
    """
    login_ok = 0
    user = "toto"
    while login_ok == 0 and user != "":
        user = raw_input("Entrez votre login zephir (rien pour sortir) : ")
        if user != "":
            passwd = getpass.getpass("Mot de passe zephir pour %s : " % user)
            # création du proxy avec zephir
            proxy = EoleProxy("https://%s:%s@%s:%s" % (user, passwd, addr, port),transport=TransportEole())
            login_ok = 1
            try:
                res=convert(proxy.get_permissions(user))
            except xmlrpclib.ProtocolError:
                login_ok = 0
                print "\n Erreur d'authentification \n"
        else:
            return False, "! Abandon de la procédure !"
    return True, proxy


if __name__ == '__main__':
    if os.path.isfile('/etc/init.d/zephir'):
        is_zephir = True
    else:
        is_zephir = False
    if registered:
        if not is_zephir:
            os.system('/etc/init.d/z_stats stop >/dev/null')
        else:
            os.system('/etc/init.d/zephir stop >/dev/null')
        # on bloque les fonctions zephir pendant la procédure
        lock(['maj','reconfigure'])
        # upgrade des infos enregistrées sur zephir pour ce client
        res = upgrade()
        unlock(['maj','reconfigure'])
        if res == "NO CONF":
            # Serveur non upgradé, on ne relance pas z_stats
            # (enregistrement_zephir nécessaire)
            sys.exit(0)
        elif res not in ["", "NO CONF"]:
            print res
        if not is_zephir:
            os.system('/etc/init.d/z_stats start >/dev/null')
        else:
            os.system('/etc/init.d/zephir start >/dev/null')
        if res not in ["", "NO CONF"]:
            sys.exit(1)
    # cas d'un upgrade sur zephir : mise à niveau des modules
    if is_zephir:
        # redémarrage de postgresql et zephir
        os.system('/etc/init.d/zephir stop')
        os.system('/etc/init.d/postgresql-8.3 stop')
        os.system('/etc/init.d/postgresql-8.3 start')
        os.system('/etc/init.d/zephir start')
        # attente du démarrage du service
        service_ok = False
        retries = 0
        print "\nMise en place des variantes sur les nouveaux modules (application zephir locale)"
        print "\nCette fonctionnalité nécessite un compte ayant la permission de créer des variantes"
        while not service_ok:
            try:
                # migration des variantes de la version précédente
                code, result = upgrade_modules(old_version, new_version)
                if not code:
                    print "\nErreur : %s\n" % result
                service_ok = True
            except:
                if retries == 0:
                    print "Echec, attente du démarrage du service ..."
                if retries <= 5:
                    retries += 1
                    time.sleep(4)
                else:
                    print "Erreur, le service zephir ne répond pas"
                    service_ok = True
