# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2011
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent FreshClam
"""
from pyeole.diagnose import compare_iptables, compare_ipset

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import TableData
from zephir.monitor.agentmanager import status
from os.path import isfile
from zephir.monitor.agentmanager.util import status_to_img
import time
import os

reports_file = '/var/lib/eole/reports/bastion.log'

class Bastion(Agent):
    """
    Utilisateurs connectes
    """
    def __init__(self, name, **params):
        Agent.__init__(self, name, **params)
        self.status = status.Unknown()
        self.table = TableData([
            ('status', 'état', {'align':'center'}, status_to_img),
            ('msg', 'message', {'align':'left'}, None)
        ])
        self.data = [self.table]

    def measure(self):
        ret = {}
        try:
            if isfile(reports_file) and os.stat(reports_file).st_size != 0:
                execfile(reports_file, {}, ret)
                if ret['STATUS'] == '0':
                    ipt = int(compare_iptables())
                    if ipt == 2:
                        msg = "règles iptables non exportées"
                        self.status = status.Error()
                        lstatus = 'Off'
                    elif ipt == 1:
                        msg = "règles iptables incorrectes"
                        self.status = status.Error()
                        lstatus = 'Off'
                    else:
                        ips = int(compare_ipset())
                        if ipt == 0:
                            last_update = time.strftime("%d %b %Y %H:%M:%S", time.localtime(float(ret['DATE'])))
                            msg = "Dernière génération des règles bastion {0}".format(last_update)
                            self.status = status.OK()
                            lstatus = 'On'
                        else:
                            msg = "règles ipset incorrectes"
                            self.status = status.Error()
                            lstatus = 'Off'
                else:
                    msg = ret['MSG']
                    self.status = status.Error()
                    lstatus = 'Off'

            else:
                self.status = status.Unknown()
                lstatus = ''
                msg = "Le fichier %s n'existe pas ou est vide" % reports_file
        except Exception, e:
            import traceback
            traceback.print_exc()
            self.status = status.Error()
            lstatus = 'Off'
            msg = "Impossible de lire %s : %s" % (reports_file, e)
        return [{'msg': msg, 'status': lstatus}]

    def check_status(self):
        return self.status

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value
